@@ -1,19 +1,23 @@
-import webbrowser
+from python_framework.api.src.service import WebBrowser
 from python_helper import Constant as c
-from python_helper import log, StringHelper
+from python_helper import log, StringHelper, Function, ReflectionHelper, ObjectHelper, SettingHelper
 from flask import Response, request
 import flask_restful
-from python_framework.api.src.annotation.MethodWrapper import Function, FunctionThrough, overrideSignatures
+from python_framework.api.src.enumeration.HttpStatus import HttpStatus
 from python_framework.api.src.helper import Serializer
 from python_framework.api.src.service import GlobalException
-from python_framework.api.src.domain import HttpStatus
 from python_framework.api.src.service import Security
 from python_framework.api.src.service.openapi import OpenApiManager
+import globals
+import json
+
+FRAMEWORK_GLOBALS_INSTANCE = None
 
 KW_URL = 'url'
 KW_DEFAULT_URL = 'defaultUrl'
 KW_MODEL = 'model'
 KW_API = 'api'
+KW_APP = 'app'
 
 KW_METHOD = 'method'
 
@@ -38,48 +42,127 @@ KW_RESOURCE_LIST = [
     KW_CONVERTER_RESOURCE
 ]
 
-LOCALHOST_URL = 'http://127.0.0.1:5000'
-
-DOT_SPACE_CAUSE = f'''{c.DOT_SPACE}{c.LOG_CAUSE}'''
-
-def printMyStuff(stuff):
-    print()
-    print(f'    type(stuff).__name__ = {type(stuff).__name__}')
-    print(f'    type(stuff).__class__.__name__ = {type(stuff).__class__.__name__}')
-    print(f'    stuff.__class__.__name__ = {stuff.__class__.__name__}')
-    print(f'    stuff.__class__.__module__ = {stuff.__class__.__module__}')
-    print(f'    stuff.__class__.__qualname__ = {stuff.__class__.__qualname__}')
-
-def printClass(Class) :
-    print(f'{2 * c.TAB}Class.__name__ = {Class.__name__}')
-    print(f'{2 * c.TAB}Class.__module__ = {Class.__module__}')
-    print(f'{2 * c.TAB}Class.__qualname__ = {Class.__qualname__}')
+def runGlobals(
+    filePath
+    , successStatus = False
+    , settingStatus = False
+    , debugStatus = False
+    , warningStatus = False
+    , wrapperStatus = False
+    , failureStatus = False
+    , errorStatus = False
+    , testStatus = False
+    , logStatus = False
+) :
+    return globals.newGlobalsInstance(
+        filePath
+        , successStatus = successStatus
+        , settingStatus = settingStatus
+        , debugStatus = debugStatus
+        , warningStatus = warningStatus
+        , wrapperStatus = wrapperStatus
+        , failureStatus = failureStatus
+        , errorStatus = errorStatus
+        , testStatus = testStatus
+        , logStatus = logStatus
+    )
 
 @Function
-def isPresent(object) :
-    if object or isinstance(type(object), dict) or {} == object :
-        return True
-    return False
+def initialize(
+    apiInstance
+    , defaultUrl = None
+    , openInBrowser = False
+    , filePath = None
+    , successStatus = False
+    , settingStatus = False
+    , debugStatus = False
+    , warningStatus = False
+    , wrapperStatus = False
+    , failureStatus = False
+    , errorStatus = False
+    , testStatus = False
+    , logStatus = False
+) :
+    if ObjectHelper.isNone(apiInstance) :
+        globalsInstance = globals.newGlobalsInstance(
+            filePath
+            , successStatus = successStatus
+            , settingStatus = settingStatus
+            , debugStatus = debugStatus
+            , warningStatus = warningStatus
+            , wrapperStatus = wrapperStatus
+            , failureStatus = failureStatus
+            , errorStatus = errorStatus
+            , testStatus = testStatus
+            , logStatus = logStatus
+        )
+    defaultUrl
+    openInBrowser
+    url = f'{apiInstance.scheme}://{apiInstance.host}{c.NOTHING if ObjectHelper.isEmpty(apiInstance.port) else f"{c.COLON}{apiInstance.port}"}{apiInstance.baseUrl}'
+    if defaultUrl :
+        url = f'{url}{defaultUrl}'
+    def inBetweenFunction(function,*argument,**keywordArgument) :
+        log.debug(initialize,f'''{function.__name__} method''')
+        if (openInBrowser) :
+            log.debug(initialize,f'''Openning "{url}" url in rowser''')
+            # WebBrowser.openUrlInChrome(url)
+            WebBrowser.openUrl(url)
+        def innerFunction(*args,**kwargs) :
+            try :
+                functionReturn = function(*args,**kwargs)
+            except Exception as exception :
+                raise Exception(f'Failed to initialize. Cause: {str(exception)}')
+            return functionReturn
+        return innerFunction
+    return inBetweenFunction
 
 @Function
-def jsonifyResponse(object, contentType, status) :
-    return Response(Serializer.jsonifyIt(object),  mimetype = contentType, status = status)
+def getRequestBodyAsJson(contentType) :
+    try :
+        if OpenApiManager.DEFAULT_CONTENT_TYPE == contentType :
+            requestBodyAsJson = request.get_json()
+        elif OpenApiManager.MULTIPART_X_MIXED_REPLACE in contentType :
+            requestBodyAsJson = request.get_data()
+        else :
+            raise Exception(f'Content type "{contentType}" not implemented')
+    except Exception as exception :
+        raise GlobalException.GlobalException(message='Not possible to parse the request', logMessage=str(exception), status=HttpStatus.BAD_REQUEST)
+    return requestBodyAsJson
 
 @Function
-def getClassName(instance) :
-    return instance.__class__.__name__
+@Security.jwtRequired
+def securedControllerMethod(args, kwargs, contentType, resourceInstance, resourceInstanceMethod, roleRequired, requestClass, logRequest) :
+    if not Security.getRole() in roleRequired :
+        raise GlobalException.GlobalException(message='Role not allowed', logMessage=f'''Role {Security.getRole()} trying to access denied resourse''', status=HttpStatus.FORBIDEN)
+    return publicControllerMethod(args, kwargs, contentType, resourceInstance, resourceInstanceMethod, requestClass, logRequest)
 
 @Function
-def getModuleName(instance) :
-    return instance.__class__.__module__
+def publicControllerMethod(args, kwargs, contentType, resourceInstance, resourceInstanceMethod, requestClass, logRequest) :
+    if resourceInstanceMethod.__name__ in OpenApiManager.ABLE_TO_RECIEVE_BODY_LIST and requestClass :
+        requestBodyAsJson = getRequestBodyAsJson(contentType)
+        if logRequest :
+            print(f'logRequest: {logRequest}')
+            log.printDebug(
+                f'"bodyRequest" : {StringHelper.prettyJson(requestBodyAsJson, withColors=SettingHelper.activeEnvironmentIsLocal())}',
+                condition = logRequest,
+                margin = False,
+                newLine = False
+            )
+        if Serializer.requestBodyIsPresent(requestBodyAsJson) :
+            serializerReturn = Serializer.convertFromJsonToObject(requestBodyAsJson, requestClass)
+            args = getArgsWithSerializerReturnAppended(serializerReturn, args, isControllerMethod=True)
+    response = resourceInstanceMethod(resourceInstance,*args[1:],**kwargs)
+    if response and Serializer.isSerializerCollection(response) and 2 == len(response) :
+        return response
+    raise GlobalException.GlobalException(logMessage=f'''Bad implementation of {resourceInstance.__class__.__name__}.{resourceInstanceMethod.__class__.__name__}() controller method''')
 
 @Function
-def getQualitativeName(instance) :
-    return instance.__class__.__qualname__
+def jsonifyResponse(response, contentType, status) :
+    return Response(Serializer.jsonifyIt(response),  mimetype=contentType, status=status)
 
 @Function
 def appendArgs(args, argument, isControllerMethod=False) :
-    if isControllerMethod and Serializer.isList(argument) :
+    if isControllerMethod and Serializer.isSerializerList(argument) :
         return args + argument
     args.append(argument)
     return args
@@ -116,142 +199,34 @@ def getResourceType(resourceInstance, resourceName = None) :
         if resourceName.endswith(resourceType):
             return resourceType
 
-@Function
-def getAttributePointerList(object) :
-    return [
-        getattr(object, objectAttributeName)
-        for objectAttributeName in dir(object)
-        if (not objectAttributeName.startswith('__') and not objectAttributeName.startswith('_'))
-    ]
-
-@Function
-def getAttributeDataList(object) :
-    return [
-        (getattr(object, objectAttributeName), objectAttributeName)
-        for objectAttributeName in dir(object)
-        if (not objectAttributeName.startswith('__') and not objectAttributeName.startswith('_'))
-    ]
-
-@Function
-def setMethod(resourceInstance, newMethod, methodName = None) :
-    def buildNewClassMethod(resourceInstance, newMethod) :
-        def myInnerMethod(*args, **kwargs) :
-            return newMethod(resourceInstance,*args, **kwargs)
-        overrideSignatures(myInnerMethod, newMethod)
-        return myInnerMethod
-    if not type(newMethod).__name__ == KW_METHOD :
-        newMethod = buildNewClassMethod(resourceInstance, newMethod)
-    if not methodName :
-        methodName = newMethod.__name__
-    setattr(resourceInstance, methodName, newMethod)
-    return resourceInstance
-
-@FunctionThrough
-def getGlobals() :
-    try :
-        from app import globals
-    except Exception as exception :
-        raise Exception('Failed to get "globals" instance from app.py')
-    return globals
-
-@FunctionThrough
-def getApi() :
-    try:
-        api = getGlobals().api
-    except Exception as exception :
-        raise Exception(f'Failed to return api from "globals" instance. Cause: {str(exception)}')
-    return api
-
-@FunctionThrough
-def getNullableApi() :
-    try :
-        api = getApi()
-    except :
-        api = None
-    return api
-
-@FunctionThrough
-def raiseBadResponseImplementetion(cause):
-    raise Exception(f'Bad response implementation. {cause}')
-
-@FunctionThrough
-def validateFlaskApi(instance) :
-    apiClassName = flask_restful.Api.__name__
-    moduleName = flask_restful.__name__
-    if not apiClassName == getClassName(instance) and apiClassName == getQualitativeName(instance) and moduleName == getModuleName(instance) :
-        raise Exception(f'Globals can only be added to a "flask_restful.Api" instance. Not to {apiInstance}')
-
-@FunctionThrough
-def validateResponseClass(responseClass, controllerResponse) :
-    if responseClass :
-        if not isPresent(controllerResponse) and not isinstance(controllerResponse, list):
-            raiseBadResponseImplementetion(f'Response not present')
-        if isinstance(responseClass, list) :
-            if 0 == len(responseClass) :
-                raiseBadResponseImplementetion(f'"responseClass" was not defined')
-            elif len(responseClass) == 1 :
-                if not isinstance(responseClass[0], list)  :
-                    if not isinstance(controllerResponse, responseClass[0]) :
-                        raiseBadResponseImplementetion(f'Response class does not match expected class. Expected "{responseClass[0].__name__}", response "{controllerResponse.__class__.__name__}"')
-                elif not isinstance(responseClass[0][0], list) :
-                    if not isinstance(controllerResponse, list)  :
-                        raiseBadResponseImplementetion(f'Response is not a list. Expected "{responseClass[0].__class__.__name__}", but found "{controllerResponse.__class__.__name__}"')
-                    elif isinstance(controllerResponse, list) and len(controllerResponse) > 0 and not isinstance(controllerResponse[0], responseClass[0][0]):
-                        # print(f'responseClass = {responseClass}')
-                        # print(f'responseClass[0] = {responseClass[0]}')
-                        # print(f'responseClass[0][0] = {responseClass[0][0]}')
-                        raiseBadResponseImplementetion(f'Response element class does not match expected element class. Expected "{responseClass[0][0].__name__}", response "{controllerResponse[0].__class__.__name__}"')
-        else :
-            if not isinstance(controllerResponse, responseClass) :
-                raiseBadResponseImplementetion(f'Response class does not match expected class. Expected "{responseClass.__name__}", response "{controllerResponse.__class__.__name__}"')
-
-
 @Function
 def setResource(apiInstance, resourceInstance, resourceName=None) :
     resourceName = getResourceFinalName(resourceInstance, resourceName=resourceName)
-    setattr(apiInstance,resourceName,resourceInstance)
+    ReflectionHelper.setAttributeOrMethod(apiInstance,resourceName,resourceInstance)
 
 @Function
 def bindResource(apiInstance,resourceInstance) :
     validateFlaskApi(apiInstance)
     setResource(getattr(apiInstance.resource, getResourceType(resourceInstance).lower()), resourceInstance)
 
-@FunctionThrough
-def getGlobalException(exception, resourceInstance, resourceInstanceMethod):
-    apiInstance = getNullableApi()
-    return GlobalException.handleLogErrorException(exception, resourceInstance, resourceInstanceMethod, apiInstance)
-
-@FunctionThrough
-def raiseGlobalException(exception, resourceInstance, resourceInstanceMethod) :
-    raise getGlobalException(exception, resourceInstance, resourceInstanceMethod)
-
-@FunctionThrough
-def getCompleteResponseByException(exception, resourceInstance, resourceInstanceMethod) :
-    exception = getGlobalException(exception, resourceInstance, resourceInstanceMethod)
-    completeResponse = [{'message':exception.message, 'timestamp':str(exception.timeStamp)},exception.status]
-    log.error(resourceInstance.__class__, f'Error processing {resourceInstance.__class__.__name__}.{resourceInstanceMethod.__name__} request', exception)
-    return completeResponse
-
-@FunctionThrough
-def initialize(apiInstance, defaultUrl=None, openInBrowser=False) :
-    defaultUrl = defaultUrl
-    openInBrowser = openInBrowser
-    url = f'{apiInstance.host}{apiInstance.baseUrl}'
-    if defaultUrl :
-        url = f'{url}{defaultUrl}'
-    def inBetweenFunction(function,*argument,**keywordArgument) :
-        log.debug(initialize,f'''{function.__name__} method''')
-        if (openInBrowser) :
-            log.debug(initialize,f'''Openning "{url}" url in rowser''')
-            webbrowser.open_new(url)
-        def innerFunction(*args,**kwargs) :
-            try :
-                functionReturn = function(*args,**kwargs)
-            except Exception as exception :
-                raise Exception(f'Failed to initialize. Cause: {str(exception)}')
-            return functionReturn
-        return innerFunction
-    return inBetweenFunction
+@Function
+def validateArgs(args, requestClass, method) :
+    if requestClass :
+        resourceInstance = args[0]
+        if Serializer.isSerializerList(requestClass) :
+            for index in range(len(requestClass)) :
+                if Serializer.isSerializerList(args[index + 1]) and len(args[index + 1]) > 0 :
+                    expecteObjectClass = requestClass[index][0]
+                    for objectInstance in args[index + 1] :
+                        GlobalException.validateArgs(resourceInstance, method, objectInstance, expecteObjectClass)
+                else :
+                    objectRequest = args[index + 1]
+                    expecteObjectClass = requestClass[index]
+                    GlobalException.validateArgs(resourceInstance, method, objectRequest, expecteObjectClass)
+        else :
+            objectRequest = args[1]
+            expecteObjectClass = requestClass
+            GlobalException.validateArgs(resourceInstance, method, objectRequest, expecteObjectClass)
 
 @Function
 def Controller(
@@ -263,56 +238,20 @@ def Controller(
     controllerTag = tag
     controllerDescription = description
     def Wrapper(OuterClass,*args,**kwargs):
-        apiInstance = getApi()
-        log.debug(Controller,f'''wrapping {OuterClass.__name__}''')
+        log.wraper(Controller, f'''wrapping {OuterClass.__name__}''', None)
         class InnerClass(OuterClass,flask_restful.Resource):
             url = controllerUrl
             tag = controllerTag
             description = controllerDescription
             def __init__(self,*args,**kwargs):
-                log.debug(OuterClass,f'in {InnerClass.__name__}.__init__(*{args},**{kwargs})')
+                log.wraper(OuterClass, f'in {InnerClass.__name__}.__init__(*{args},**{kwargs})', None)
                 OuterClass.__init__(self)
                 flask_restful.Resource.__init__(self,*args,**kwargs)
-                self.service = apiInstance.resource.service
-        overrideSignatures(InnerClass, OuterClass)
+                self.service = getApi().resource.service
+        ReflectionHelper.overrideSignatures(InnerClass, OuterClass)
         return InnerClass
     return Wrapper
 
-@FunctionThrough
-def getRequestBodyAsJson(contentType) :
-    try :
-        if OpenApiManager.DEFAULT_CONTENT_TYPE == contentType :
-            requestBodyAsJson = request.get_json()
-        elif 'multipart/x-mixed-replace' in contentType :
-            requestBodyAsJson = request.get_data()
-        else :
-            raise Exception(f'Content type "{contentType}" not implemented')
-    except Exception as exception :
-        raise GlobalException.GlobalException(message='Not possible to parse the request', logMessage=str(exception), status=HttpStatus.BAD_REQUEST)
-    return requestBodyAsJson
-
-@FunctionThrough
-@Security.jwtRequired
-def securedControllerMethod(args, kwargs, contentType, resourceInstance, resourceInstanceMethod, roleRequired, requestClass, logRequest) :
-    if not Security.getRole() in roleRequired :
-        raise GlobalException.GlobalException(message='Role not allowed', logMessage=f'''Role {Security.getRole()} trying to access denied resourse''', status=HttpStatus.FORBIDEN)
-    return publicControllerMethod(args, kwargs, contentType, resourceInstance, resourceInstanceMethod, requestClass, logRequest)
-
-
-@FunctionThrough
-def publicControllerMethod(args, kwargs, contentType, resourceInstance, resourceInstanceMethod, requestClass, logRequest) :
-    if resourceInstanceMethod.__name__ in OpenApiManager.ABLE_TO_RECIEVE_BODY_LIST and requestClass :
-        requestBodyAsJson = getRequestBodyAsJson(contentType)
-        if logRequest :
-            log.debug(resourceInstanceMethod, f'"bodyRequest" : {Serializer.prettify(requestBodyAsJson)}')
-        if  isPresent(requestBodyAsJson) :
-            serializerReturn = Serializer.convertFromJsonToObject(requestBodyAsJson, requestClass)
-            args = getArgsWithSerializerReturnAppended(serializerReturn, args, isControllerMethod=True)
-    response = resourceInstanceMethod(resourceInstance,*args[1:],**kwargs)
-    if response and Serializer.isCollection(response) and 2 == len(response) :
-        return response
-    raise GlobalException.GlobalException(logMessage=f'''Bad implementation of {resourceInstance.__class__.__name__}.{resourceInstanceMethod.__class__.__name__}() controller method''')
-
 @Function
 def ControllerMethod(
     url = c.SLASH,
@@ -324,6 +263,7 @@ def ControllerMethod(
     logRequest = False,
     logResponse = False
 ):
+
     controllerMethodUrl = url
     controllerMethodRequestClass = requestClass
     controllerMethodResponseClass = responseClass
@@ -333,17 +273,15 @@ def ControllerMethod(
     controllerMethodLogRequest = logRequest
     controllerMethodLogResponse = logResponse
     def innerMethodWrapper(resourceInstanceMethod,*args,**kwargs) :
-        noException = None
-        log.debug(ControllerMethod,f'''wrapping {resourceInstanceMethod.__name__}''')
+        log.wraper(ControllerMethod, f'''wrapping {resourceInstanceMethod.__name__}''', None)
         def innerResourceInstanceMethod(*args,**kwargs) :
             resourceInstance = args[0]
             try :
-                if roleRequired and (type(list()) == type(roleRequired) and not [] == roleRequired) :
+                if ObjectHelper.isNotEmptyCollection(roleRequired) :
                     completeResponse = securedControllerMethod(args, kwargs, consumes, resourceInstance, resourceInstanceMethod, roleRequired, requestClass, logRequest)
                 else :
                     completeResponse = publicControllerMethod(args, kwargs, consumes, resourceInstance, resourceInstanceMethod, requestClass, logRequest)
-                validateResponseClass(responseClass, completeResponse[0])
-
+                validateResponseClass(responseClass, completeResponse)
             except Exception as exception :
                 completeResponse = getCompleteResponseByException(exception, resourceInstance, resourceInstanceMethod)
                 ###- request.method:              GET
@@ -359,13 +297,17 @@ def ControllerMethod(
                 ###- request.full_path:           /alert/dingding/test?x=y
                 ###- request.args:                ImmutableMultiDict([('x', 'y')])
                 ###- request.args.get('x'):       y
-
             controllerResponse = completeResponse[0]
             status = completeResponse[1]
             if logResponse :
-                log.debug(innerResourceInstanceMethod, f'"bodyResponse" : {Serializer.prettify(controllerResponse)}')
+                log.printDebug(
+                    f'"bodyResponse" : {StringHelper.prettyJson(json.loads(Serializer.jsonifyIt(controllerResponse)), withColors=SettingHelper.activeEnvironmentIsLocal())}',
+                    condition = logResponse,
+                    margin = False,
+                    newLine = False
+                )
             return jsonifyResponse(controllerResponse, produces, status)
-        overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
+        ReflectionHelper.overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
         innerResourceInstanceMethod.url = controllerMethodUrl
         innerResourceInstanceMethod.requestClass = controllerMethodRequestClass
         innerResourceInstanceMethod.responseClass = controllerMethodResponseClass
@@ -377,35 +319,15 @@ def ControllerMethod(
         return innerResourceInstanceMethod
     return innerMethodWrapper
 
-@Function
-def validateArgs(args, requestClass, method) :
-    if requestClass :
-        resourceInstance = args[0]
-        if Serializer.isList(requestClass) :
-            for index in range(len(requestClass)) :
-                if Serializer.isList(args[index + 1]) and len(args[index + 1]) > 0 :
-                    expecteObjectClass = requestClass[index][0]
-                    for objectInstance in args[index + 1] :
-                        GlobalException.validateArgs(resourceInstance, method, objectInstance, expecteObjectClass)
-                else :
-                    objectRequest = args[index + 1]
-                    expecteObjectClass = requestClass[index]
-                    GlobalException.validateArgs(resourceInstance, method, objectRequest, expecteObjectClass)
-        else :
-            objectRequest = args[1]
-            expecteObjectClass = requestClass
-            GlobalException.validateArgs(resourceInstance, method, objectRequest, expecteObjectClass)
-
 @Function
 def Service() :
     def Wrapper(OuterClass, *args, **kwargs):
-        apiInstance = getApi()
-        noException = None
         log.debug(Service,f'''wrapping {OuterClass.__name__}''')
         class InnerClass(OuterClass):
             def __init__(self,*args,**kwargs):
                 log.debug(OuterClass,f'in {InnerClass.__name__}.__init__(*{args},**{kwargs})')
                 OuterClass.__init__(self,*args,**kwargs)
+                apiInstance = getApi()
                 self.globals = apiInstance.globals
                 self.service = apiInstance.resource.service
                 self.client = apiInstance.resource.client
@@ -414,14 +336,13 @@ def Service() :
                 self.mapper = apiInstance.resource.mapper
                 self.helper = apiInstance.resource.helper
                 self.converter = apiInstance.resource.converter
-        overrideSignatures(InnerClass, OuterClass)
+        ReflectionHelper.overrideSignatures(InnerClass, OuterClass)
         return InnerClass
     return Wrapper
 
 @Function
 def ServiceMethod(requestClass=None):
     def innerMethodWrapper(resourceInstanceMethod,*args,**kwargs) :
-        noException = None
         log.debug(ServiceMethod,f'''innerMethodWrapper wraped {resourceInstanceMethod.__name__}''')
         def innerResourceInstanceMethod(*args,**kwargs) :
             resourceInstance = args[0]
@@ -431,29 +352,26 @@ def ServiceMethod(requestClass=None):
             except Exception as exception :
                 raiseGlobalException(exception, resourceInstance, resourceInstanceMethod)
             return methodReturn
-        overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
+        ReflectionHelper.overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
         return innerResourceInstanceMethod
     return innerMethodWrapper
 
 @Function
 def Client() :
     def Wrapper(OuterClass, *args, **kwargs):
-        apiInstance = getApi()
-        noException = None
         log.debug(Client,f'''wrapping {OuterClass.__name__}''')
         class InnerClass(OuterClass):
             def __init__(self,*args,**kwargs):
                 log.debug(OuterClass,f'in {InnerClass.__name__}.__init__(*{args},**{kwargs})')
                 OuterClass.__init__(self,*args,**kwargs)
-                self.globals = apiInstance.globals
-        overrideSignatures(InnerClass, OuterClass)
+                self.globals = getApi().globals
+        ReflectionHelper.overrideSignatures(InnerClass, OuterClass)
         return InnerClass
     return Wrapper
 
 @Function
 def ClientMethod(requestClass=None):
     def innerMethodWrapper(resourceInstanceMethod,*args,**kwargs) :
-        noException = None
         log.debug(ClientMethod,f'''innerMethodWrapper wraped {resourceInstanceMethod.__name__}''')
         def innerResourceInstanceMethod(*args,**kwargs) :
             resourceInstance = args[0]
@@ -463,7 +381,7 @@ def ClientMethod(requestClass=None):
             except Exception as exception :
                 raiseGlobalException(exception, resourceInstance, resourceInstanceMethod)
             return methodReturn
-        overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
+        ReflectionHelper.overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
         return innerResourceInstanceMethod
     return innerMethodWrapper
 
@@ -471,42 +389,39 @@ def ClientMethod(requestClass=None):
 def Repository(model = None) :
     repositoryModel = model
     def Wrapper(OuterClass, *args, **kwargs):
-        apiInstance = getApi()
-        noException = None
         log.debug(Repository,f'''wrapping {OuterClass.__name__}''')
         class InnerClass(OuterClass):
             model = repositoryModel
             def __init__(self,*args,**kwargs):
                 log.debug(OuterClass,f'in {InnerClass.__name__}.__init__(*{args},**{kwargs})')
                 OuterClass.__init__(self,*args,**kwargs)
+                apiInstance = getApi()
                 self.repository = apiInstance.repository
                 self.globals = apiInstance.globals
-        overrideSignatures(InnerClass, OuterClass)
+        ReflectionHelper.overrideSignatures(InnerClass, OuterClass)
         return InnerClass
     return Wrapper
 
 @Function
 def Validator() :
     def Wrapper(OuterClass, *args, **kwargs):
-        apiInstance = getApi()
-        noException = None
         log.debug(Validator,f'''wrapping {OuterClass.__name__}''')
         class InnerClass(OuterClass):
             def __init__(self,*args,**kwargs):
                 log.debug(OuterClass,f'in {InnerClass.__name__}.__init__(*{args},**{kwargs})')
                 OuterClass.__init__(self,*args,**kwargs)
+                apiInstance = getApi()
                 self.service = apiInstance.resource.service
                 self.validator = apiInstance.resource.validator
                 self.helper = apiInstance.resource.helper
                 self.converter = apiInstance.resource.converter
-        overrideSignatures(InnerClass, OuterClass)
+        ReflectionHelper.overrideSignatures(InnerClass, OuterClass)
         return InnerClass
     return Wrapper
 
 @Function
 def ValidatorMethod(requestClass=None, message=None, logMessage=None) :
     def innerMethodWrapper(resourceInstanceMethod,*args,**kwargs) :
-        noException = None
         log.debug(ValidatorMethod,f'''wrapping {resourceInstanceMethod.__name__}''')
         def innerResourceInstanceMethod(*args,**kwargs) :
             resourceInstance = args[0]
@@ -516,7 +431,7 @@ def ValidatorMethod(requestClass=None, message=None, logMessage=None) :
             except Exception as exception :
                 raiseGlobalException(exception, resourceInstance, resourceInstanceMethod)
             return methodReturn
-        overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
+        ReflectionHelper.overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
         return innerResourceInstanceMethod
     return innerMethodWrapper
 
@@ -524,26 +439,24 @@ def ValidatorMethod(requestClass=None, message=None, logMessage=None) :
 @Function
 def Mapper() :
     def Wrapper(OuterClass, *args, **kwargs):
-        apiInstance = getApi()
-        noException = None
         log.debug(Mapper,f'''wrapping {OuterClass.__name__}''')
         class InnerClass(OuterClass):
             def __init__(self,*args,**kwargs):
                 log.debug(OuterClass,f'in {InnerClass.__name__}.__init__(*{args},**{kwargs})')
                 OuterClass.__init__(self,*args,**kwargs)
+                apiInstance = getApi()
                 self.service = apiInstance.resource.service
                 self.validator = apiInstance.resource.validator
                 self.mapper = apiInstance.resource.mapper
                 self.helper = apiInstance.resource.helper
                 self.converter = apiInstance.resource.converter
-        overrideSignatures(InnerClass, OuterClass)
+        ReflectionHelper.overrideSignatures(InnerClass, OuterClass)
         return InnerClass
     return Wrapper
 
 @Function
 def MapperMethod(requestClass=None, responseClass=None) :
     def innerMethodWrapper(resourceInstanceMethod,*args,**kwargs) :
-        noException = None
         log.debug(MapperMethod,f'''wrapping {resourceInstanceMethod.__name__}''')
         def innerResourceInstanceMethod(*args,**kwargs) :
             resourceInstance = args[0]
@@ -554,30 +467,28 @@ def MapperMethod(requestClass=None, responseClass=None) :
             except Exception as exception :
                 raiseGlobalException(exception, resourceInstance, resourceInstanceMethod)
             return methodReturn
-        overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
+        ReflectionHelper.overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
         return innerResourceInstanceMethod
     return innerMethodWrapper
 
 @Function
 def Helper() :
     def Wrapper(OuterClass, *args, **kwargs):
-        apiInstance = getApi()
-        noException = None
         log.debug(Helper,f'''wrapping {OuterClass.__name__}''')
         class InnerClass(OuterClass,flask_restful.Resource):
             def __init__(self,*args,**kwargs):
                 log.debug(OuterClass,f'in {InnerClass.__name__}.__init__(*{args},**{kwargs})')
                 OuterClass.__init__(self,*args,**kwargs)
+                apiInstance = getApi()
                 self.helper = apiInstance.resource.helper
                 self.converter = apiInstance.resource.converter
-        overrideSignatures(InnerClass, OuterClass)
+        ReflectionHelper.overrideSignatures(InnerClass, OuterClass)
         return InnerClass
     return Wrapper
 
 @Function
 def HelperMethod(requestClass=None, responseClass=None) :
     def innerMethodWrapper(resourceInstanceMethod,*args,**kwargs) :
-        noException = None
         log.debug(HelperMethod,f'''wrapping {resourceInstanceMethod.__name__}''')
         def innerResourceInstanceMethod(*args,**kwargs) :
             resourceInstance = args[0]
@@ -588,30 +499,28 @@ def HelperMethod(requestClass=None, responseClass=None) :
             except Exception as exception :
                 raiseGlobalException(exception, resourceInstance, resourceInstanceMethod)
             return methodReturn
-        overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
+        ReflectionHelper.overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
         return innerResourceInstanceMethod
     return innerMethodWrapper
 
 @Function
 def Converter() :
     def Wrapper(OuterClass, *args, **kwargs):
-        apiInstance = getApi()
-        noException = None
         log.debug(Converter,f'''wrapping {OuterClass.__name__}''')
         class InnerClass(OuterClass):
             def __init__(self,*args,**kwargs):
                 log.debug(OuterClass,f'in {InnerClass.__name__}.__init__(*{args},**{kwargs})')
                 OuterClass.__init__(self,*args,**kwargs)
+                apiInstance = getApi()
                 self.helper = apiInstance.resource.helper
                 self.converter = apiInstance.resource.converter
-        overrideSignatures(InnerClass, OuterClass)
+        ReflectionHelper.overrideSignatures(InnerClass, OuterClass)
         return InnerClass
     return Wrapper
 
 @Function
 def ConverterMethod(requestClass=None, responseClass=None) :
     def innerMethodWrapper(resourceInstanceMethod,*args,**kwargs) :
-        noException = None
         log.debug(ConverterMethod,f'''wrapping {resourceInstanceMethod.__name__}''')
         def innerResourceInstanceMethod(*args,**kwargs) :
             resourceInstance = args[0]
@@ -622,6 +531,85 @@ def ConverterMethod(requestClass=None, responseClass=None) :
             except Exception as exception :
                 raiseGlobalException(exception, resourceInstance, resourceInstanceMethod)
             return methodReturn
-        overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
+        ReflectionHelper.overrideSignatures(innerResourceInstanceMethod, resourceInstanceMethod)
         return innerResourceInstanceMethod
     return innerMethodWrapper
+
+def getGlobalException(exception, resourceInstance, resourceInstanceMethod):
+    apiInstance = getNullableApi()
+    return GlobalException.handleLogErrorException(exception, resourceInstance, resourceInstanceMethod, apiInstance)
+
+def raiseGlobalException(exception, resourceInstance, resourceInstanceMethod) :
+    raise getGlobalException(exception, resourceInstance, resourceInstanceMethod)
+
+def getCompleteResponseByException(exception, resourceInstance, resourceInstanceMethod) :
+    exception = getGlobalException(exception, resourceInstance, resourceInstanceMethod)
+    completeResponse = [{'message':exception.message, 'timestamp':str(exception.timeStamp)},exception.status]
+    log.error(resourceInstance.__class__, f'Error processing {resourceInstance.__class__.__name__}.{resourceInstanceMethod.__name__} request', exception)
+    return completeResponse
+
+def validateResponseClass(responseClass, controllerResponse) :
+    if isNotPythonFrameworkHttpsResponse(controllerResponse) :
+        raiseBadResponseImplementetion(f'Python Framework response cannot be null. It should be a list like this: [{"RESPONSE_CLASS" if ObjectHelper.isNone(responseClass) else responseClass if ObjectHelper.isNotList(responseClass) else responseClass[0]}, HTTPS_CODE]')
+    if ObjectHelper.isNotNone(responseClass) :
+        if Serializer.isSerializerList(responseClass) :
+            if 0 == len(responseClass) :
+                log.warning(validateResponseClass,f'"responseClass" was not defined')
+            elif 1 == len(responseClass) :
+                if ObjectHelper.isNotList(responseClass[0])  :
+                    if not isinstance(controllerResponse[0], responseClass[0]) :
+                        raiseBadResponseImplementetion(f'Response class does not match expected class. Expected "{responseClass[0].__name__}", response "{controllerResponse[0].__class__.__name__}"')
+                elif ObjectHelper.isNotList(responseClass[0][0]) :
+                    if ObjectHelper.isNotList(controllerResponse[0]) :
+                        raiseBadResponseImplementetion(f'Response is not a list. Expected "{responseClass[0].__class__.__name__}", but found "{controllerResponse[0].__class__.__name__}"')
+                    elif Serializer.isSerializerList(controllerResponse[0]) and 0 < len(controllerResponse[0]) and not isinstance(controllerResponse[0][0], responseClass[0][0]) :
+                        raiseBadResponseImplementetion(f'Response element class does not match expected element class. Expected "{responseClass[0][0].__name__}", response "{controllerResponse[0][0].__class__.__name__}"')
+        else :
+            if not isinstance(controllerResponse[0], responseClass) :
+                raiseBadResponseImplementetion(f'Response class does not match expected class. Expected "{responseClass.__name__}", response "{controllerResponse[0].__class__.__name__}"')
+    else :
+        log.warning(validateResponseClass,f'"responseClass" was not defined')
+
+def getClassName(instance) :
+    return instance.__class__.__name__
+
+def getModuleName(instance) :
+    return instance.__class__.__module__
+
+def getQualitativeName(instance) :
+    return instance.__class__.__qualname__
+
+def isPythonFrameworkHttpsResponse(controllerResponse) :
+    return (ObjectHelper.isTuple(controllerResponse) or ObjectHelper.isList(controllerResponse)) and 2 == len(controllerResponse)
+
+def isNotPythonFrameworkHttpsResponse(controllerResponse) :
+    return not isPythonFrameworkHttpsResponse(controllerResponse)
+
+def raiseBadResponseImplementetion(cause):
+    raise Exception(f'Bad response implementation. {cause}')
+
+@Function
+def getGlobals() :
+    return globals.getGlobalsInstance()
+
+def getApi() :
+    api = None
+    try:
+        api = getGlobals().api
+    except Exception as exception :
+        raise Exception(f'Failed to return api from "globals" instance. Cause: {str(exception)}')
+    return api
+
+def getNullableApi() :
+    api=None
+    try :
+        api = getApi()
+    except Exception as exception :
+        log.warning(getNullableApi, 'Not possible to get api', exception=exception)
+    return api
+
+def validateFlaskApi(instance) :
+    apiClassName = flask_restful.Api.__name__
+    moduleName = flask_restful.__name__
+    if not apiClassName == getClassName(instance) and apiClassName == getQualitativeName(instance) and moduleName == getModuleName(instance) :
+        raise Exception(f'Globals can only be added to a "flask_restful.Api" instance. Not to {apiInstance}')
